﻿Imports XLSXPack.ClassMyCommon ''ClassMyCoomonのプロパティ・メソッドを参照するためのImportsステートメント

''' <summary>ログ管理クラス</summary>
Public Class ClassLog

    ''' <summary>フルパスでのログファイル名</summary>
    Private LogFileName As String

    ''' <summary>ログファイル保存用の裏バッファ</summary>
    Private _ShadowBuf As String

    ''' <summary>フォーム上のログ表示用テキストボックス</summary>
    Private _LogTextBox As TextBox

    ''' <summary>コンストラクタ</summary>
    ''' <param name="LogTextBox">フォーム上のログ表示用テキストボックス</param>
    Public Sub New(ByVal LogTextBox As TextBox)
        _LogTextBox = LogTextBox
        _LogTextBox.Text = ""
        _ShadowBuf = ""
        LogFileName = AppPath() + "Log" + Now.ToString("yyMM") + ".txt"
    End Sub

    ''' <summary>Delegate ログの追加</summary>
    ''' <param name="stradd">ログ追加文字列</param>
    Delegate Sub Add_Callback(ByVal stradd As String)

    ''' <summary>ログの追加</summary>
    ''' <param name="stradd">ログ追加文字列</param>
    ''' <remarks>ログの最下行へのスクロール・表示更新は行わない</remarks>
    Public Sub Add(ByVal stradd As String)
        Dim SplitBuf() As String
        Dim i As Integer
        Dim TempText As String

        ''別スレッドか判定
        If _LogTextBox.InvokeRequired Then
            ''別スレッドからの場合コールバック処理をして終了
            Dim d As New Add_Callback(AddressOf Add)
            _LogTextBox.Invoke(d, New Object() {stradd})
            Exit Sub
        End If

        If Len(stradd) = 0 Then Exit Sub

        If InStr(stradd, vbCrLf) = 0 Then stradd &= vbCrLf

        SplitBuf = Split(stradd, vbCrLf)
        If UBound(SplitBuf) > 0 Then
            For i = 0 To UBound(SplitBuf) - 1
                'TempText = Format(Now, "yy/MM/dd HH:mm:ss") & " : " & SplitBuf(i) & vbCrLf
                TempText = Format(Now, "MM/dd HH:mm:ss") & " : " & SplitBuf(i) & vbCrLf
                _LogTextBox.Text += TempText
                _ShadowBuf += TempText
            Next i
        End If

    End Sub

    ''' <summary>Delegate ログの追加と表示の更新</summary>
    ''' <param name="stradd">ログ追加文字列</param>
    Delegate Sub Put_Callback(ByVal stradd As String)

    ''' <summary>ログの最終行を表示する。</summary>
    Public Sub Put()
        ''カレット位置を末尾に移動
        _LogTextBox.SelectionStart = _LogTextBox.Text.Length
        ''カレット位置までスクロール
        _LogTextBox.ScrollToCaret()
    End Sub

    ''' <summary>ログの追加と最下行へのスクロール・表示更新</summary>
    ''' <param name="stradd">ログ追加文字列</param>
    Public Sub Put(ByVal stradd As String)

        ''別スレッドか判定
        If _LogTextBox.InvokeRequired Then
            ''別スレッドからの場合コールバック処理をして終了
            Dim d As New Put_Callback(AddressOf Put)
            _LogTextBox.Invoke(d, New Object() {stradd})
            Exit Sub
        End If

        ''古いログを廃棄する
        DeleteOld()

        ''ログ追加処理
        Add(stradd)

        ''最新ログまでスクロール
        Put()

    End Sub

    ''' <summary>デバッグログの追加と最下行へのスクロール・表示更新</summary>
    ''' <param name="stradd">デバッグログ追加文字列</param>
    Public Sub DebugPut(ByVal stradd As String)

#If 0 Then
            Put(stradd)
#End If

    End Sub

    ''' <summary>ログをファイルへ保存する</summary>
    Public Sub Save()
        If _ShadowBuf.Length = 0 Then Exit Sub

#If 0 Then 'このプログラムでは保存機能は封印する。
        ''追記モードでShift-JISファイルを開く
        Using Writer As New System.IO.StreamWriter(LogFileName, True, System.Text.Encoding.GetEncoding("shift-jis"))
            Writer.Write(_ShadowBuf)
        End Using
#End If
        _ShadowBuf = ""
    End Sub

    ''' <summary>古いログ表示を破棄する</summary>
    Private Sub DeleteOld()
        Dim SplitBuf() As String

        ''ログ表示量が MaxLength -1024 より小さくなるまで破棄する。
        While Len(_LogTextBox.Text) > (_LogTextBox.MaxLength - 1024)
            SplitBuf = Split(_LogTextBox.Text, vbLf, 2)
            If UBound(SplitBuf) > 0 Then
                _LogTextBox.Text = SplitBuf(1)
            Else
                _LogTextBox.Text = ""
            End If
        End While
    End Sub

    ''' <summary>ログ表示を消去する。</summary>
    ''' <remarks >ファイル保存用の裏バッファはクリアしない</remarks>
    Public Sub Clear()
        _LogTextBox.Text = ""
    End Sub

End Class
