﻿Imports XLSXPack.ClassMyCommon ''ClassMyConmonのプロパティ・メソッドを参照するためのImportsステートメント
Imports System.IO
Imports System.IO.Compression

''' <summary>メイン(スタートアップ)フォーム</summary>
Public Class FormMain

    Private Sub FormMain_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        On Error Resume Next

        Log.Put("XLSXPack 終了" + vbCrLf)
        Log.Save()

    End Sub

    ''' <summary>メイン(スタートアップ)フォームのロード</summary>
    ''' <param name="sender">イベントを発生させたオブジェクト</param>
    ''' <param name="e">イベントオプション</param>
    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ''多重起動チェック
        'Process.GetProcessesByName メソッド
        '指定したプロセス名を共有するリモートコンピュータ上の
        'すべてのプロセスリソースに関連付けます。
        Dim pn As String = Process.GetCurrentProcess.ProcessName 'Process.ProcessName プロパティ(プロセスの名前を取得します)
        If Process.GetProcessesByName(pn).GetUpperBound(0) > 0 Then
            ''多重起動しているならばエラーメッセージ表示して終了。
            MessageBox.Show("すでに起動しています。", TITLE + " エラーメッセージ", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Error)
            End
        End If

        frmMain = Me

        FormControlBoxCustomize(Me)

        Me.Text = TITLE + " " + VERSION

        Log = New ClassLog(TextBox4)

        Log.DebugPut("起動しました。" + vbCrLf)

        Button4.PerformClick()

        '起動時は最前面にもってくる
        Me.Show()
        Me.TopMost = True
        WaitTickCount(500)
        Me.TopMost = False


    End Sub

    ''' <summary>「このプログラムについて」のボタンを押された時の処理</summary>
    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        Log.Add("--------------------------------------------------------------------------" + vbCrLf)
        Log.Add(TITLE + " " + VERSION + vbCrLf)
        Log.Add(OPENING_MSG)
        Log.Add(vbCrLf)
        Log.Add("--------------------------------------------------------------------------" + vbCrLf)
        Log.Put()
    End Sub

    ''' <summary>ログの表示クリアを行う</summary>
    Private Sub Button5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click
        Log.Clear()
    End Sub

    ''' <summary>プログラムの終了ボタンを押された。もしくはタスクトレイアイコンのメニューから Quit を選択された時の処理</summary>
    Private Sub Button6_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button6.Click

        Log.Put("終了します。" + vbCrLf)

        WaitTickCount(100)

        'ここで　closingイベントが発生。
        Me.Close()
        End

    End Sub

    Public Sub New()

        ' この呼び出しはデザイナーで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。

    End Sub

    Private Sub TextBox4_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles TextBox4.DragEnter

        'ファイル形式の場合のみ、ドラッグを受け付けます。
        If e.Data.GetDataPresent(DataFormats.FileDrop) = True Then
            e.Effect = DragDropEffects.Copy
        Else
            e.Effect = DragDropEffects.None
        End If

    End Sub

    Private Sub TextBox4_DragDrop(sender As Object, e As DragEventArgs) Handles TextBox4.DragDrop

        ' 読み込んだ結果をすべて格納するための変数を宣言する
        Dim Result As String
        Dim StrArray() As String

        'ドラッグされたファイル・フォルダのパスを格納します。
        Dim strFileName As String() = CType(e.Data.GetData(DataFormats.FileDrop, False), String())
        Dim XLSXFileName As String

        XLSXFileName = strFileName(0).ToString()

        'ファイルの存在確認を行い、ある場合にのみ、
        'テキストボックスにパスを表示します。
        '（この処理でフォルダを対象外にしています。）
        If System.IO.File.Exists(XLSXFileName) = True Then
            Log.Add("Drop : " + XLSXFileName)
            Log.Put("作業開始します。")
            Application.DoEvents()
        Else
            Exit Sub
        End If

        If StrComp(XLSXFileName.Substring(XLSXFileName.Length - 5), ".xlsx") Then
            Log.Put("XLSX file 以外には対応していません")
            Exit Sub
        End If

        Dim ZipFileName As String = ""
        Dim ExtractPath As String = ""

        Try
            ZipFileName = Path.GetDirectoryName(XLSXFileName) + "\XLSXPackTemp.zip"

            File.Delete(ZipFileName)
            File.Copy(XLSXFileName, ZipFileName)

            'ファイルを書き出すフォルダーを作成する
            ExtractPath = Path.GetDirectoryName(XLSXFileName) + "\XLSXPackExtract"

            ' 解凍先フォルダ (ディレクトリ) が存在しているかどうか確認して削除
            If Directory.Exists(ExtractPath) Then
                Log.DebugPut("作業用フォルダが存在します。削除します。")
                Directory.Delete(ExtractPath, True)
            Else
                Log.DebugPut("作業用フォルダが存在しません。")
            End If

            Try
                Directory.CreateDirectory(ExtractPath)
            Catch
                Log.Put("作業用フォルダの作成に失敗しました。")
                Exit Sub
            End Try

            'ZIP書庫を展開する
            Compression.ZipFile.ExtractToDirectory( _
                ZipFileName, ExtractPath,
                System.Text.Encoding.GetEncoding("shift_jis"))

            'MediaPath = .\XLSXPackExtract\xl\media
            Dim MediaPath As String
            MediaPath = ExtractPath + "\xl\media"

            Dim MediaFileNames() As String

            MediaFileNames = Directory.GetFiles(MediaPath)

            '取得したメディアファイルの処理をする

            For Each MediaFileName As String In MediaFileNames

                'ファイルのサイズを取得し小さいサイズのものはスキップ
                Dim Medialen As Long = New FileInfo(MediaFileName).Length
                If Medialen < 1024 Then
                    Continue For
                End If

                '取得したファイル名を表示する
                Log.DebugPut(MediaFileName)
                Log.DebugPut(Medialen.ToString)

                'png/bmp拡張子でなければスキップする
                Dim ExtString As String = MediaFileName.Substring(MediaFileName.Length - 4).ToLower
                If ExtString <> ".png" And ExtString <> ".bmp" Then
                    Continue For
                End If

                Dim TmpJpgFileName As String
                TmpJpgFileName = MediaPath + "XLSXPacktemp.jpg"

                'BMP/PNGファイルを開く
                Try
                    '前回の作業ファイルが残っていれば削除する
                    File.Delete(TmpJpgFileName)

                    ' Bitmapファイルオープン
                    Using bmp As New Bitmap(MediaFileName)

                        'JPEG形式で保存する()
                        bmp.Save(TmpJpgFileName, System.Drawing.Imaging.ImageFormat.Jpeg)

                        'Bitmapファイルをクローズ
                        bmp.Dispose()
                    End Using
                Catch
                    '不正なBMP/PNGであればエラーが発生する。
                    File.Delete(TmpJpgFileName)
                    Continue For
                End Try

                Dim Jpglen As Long = New FileInfo(TmpJpgFileName).Length
                If Jpglen <= (Medialen / 2) Then
                    File.Delete(MediaFileName)
                    For j = 0 To 9999
                        Dim NewMediaFileName As String
                        If j = 0 Then
                            NewMediaFileName = MediaFileName.Substring(0, MediaFileName.Length - 4) + ".jpg"
                        Else
                            NewMediaFileName = MediaFileName.Substring(0, MediaFileName.Length - 4) + "(" + (j - 1).ToString + ").jpg"
                        End If

                        Try
                            File.Move(TmpJpgFileName, NewMediaFileName)


                            StrArray = Split(MediaFileName, "\")
                            Dim SplitMediaFileName As String = StrArray(StrArray.Length - 1)
                            StrArray = Split(NewMediaFileName, "\")
                            Dim SplitNewMediaFileName As String = StrArray(StrArray.Length - 1)

                            Log.Put(SplitMediaFileName + " -> " + SplitNewMediaFileName)
                            Application.DoEvents() 'ログ表示後進の為

                            Dim RelsPath As String = ExtractPath + "\xl\drawings\_rels"
                            Dim RelsFileNames() As String
                            RelsFileNames = Directory.GetFiles(RelsPath)

                            '取得したリレーションファイルの処理をする

                            For Each RelsFileName As String In RelsFileNames
                                'Shift-JISファイルを開く
                                Result = String.Empty
                                Using Reader As New System.IO.StreamReader(RelsFileName, System.Text.Encoding.GetEncoding("shift-jis"))
                                    ' 読み込みできる文字がなくなるまで繰り返す
                                    While (Reader.Peek() >= 0)
                                        ' ファイルを 1 行ずつ読み込む
                                        Dim Buffer As String = Reader.ReadLine()
                                        ' 読み込んだものを追加で格納する
                                        Result &= Buffer & System.Environment.NewLine
                                    End While
                                End Using

                                If InStr(Result, SplitMediaFileName) Then
                                    'Log.Put(Result)
                                    Result = Replace(Result, SplitMediaFileName, SplitNewMediaFileName)
                                    '上書モードでShift-JISファイルを開く
                                    Using Writer As New System.IO.StreamWriter(RelsFileName, False, System.Text.Encoding.GetEncoding("shift-jis"))
                                        Writer.Write(Result)
                                        'Log.Put(Result)
                                    End Using

                                End If
                            Next

                            Exit For
                        Catch
                            File.Delete(NewMediaFileName)
                            Continue For
                        End Try
                    Next j
                    File.Delete(TmpJpgFileName)

                Else
                    File.Delete(TmpJpgFileName)
                End If

            Next

            Dim CurrentTypesFileName As String = ExtractPath + "\[Content_Types].xml"
            Result = String.Empty
            Using Reader As New System.IO.StreamReader(CurrentTypesFileName, System.Text.Encoding.GetEncoding("shift-jis"))
                Result = Reader.ReadToEnd
            End Using

            '追加 <Default Extension="jpg" ContentType="application/jpg"/>
            If InStr(Result.ToLower, "default extension=""jpg""") Then
                '変更不要
            Else
                Result = Replace(Result, _
                                 "<Default Extension=""xml"" ContentType=""application/xml""/>", _
                                 "<Default Extension=""xml"" ContentType=""application/xml""/><Default Extension=""jpg"" ContentType=""application/jpeg""/>")
                '上書モードでShift-JISファイルを開く
                Using Writer As New System.IO.StreamWriter(CurrentTypesFileName, False, System.Text.Encoding.GetEncoding("shift-jis"))
                    Writer.Write(Result)
                    'Log.Put(Result)
                End Using

            End If

            'ZIP書庫を作成する
            File.Delete(ZipFileName)
            Compression.ZipFile.CreateFromDirectory( _
                ExtractPath, ZipFileName,
                Compression.CompressionLevel.Optimal, _
                False, _
                System.Text.Encoding.GetEncoding("shift_jis"))

            'リネームする
            For i = 0 To 1000
                Try
                    File.Move(XLSXFileName, XLSXFileName.Substring(0, XLSXFileName.Length - 5) + "(" + i.ToString + ").xlsx")
                Catch
                    Continue For
                End Try
                Exit For
            Next

            File.Move(ZipFileName, XLSXFileName)

            '作業フォルダを削除
            Try
                Directory.Delete(ExtractPath, True)
            Catch
            End Try

            Log.Put("作業終了しました。")

        Catch
            '作業ファイルとフォルダを削除
            Try
                Directory.Delete(ExtractPath, True)
                File.Delete(ZipFileName)
            Catch

            End Try

            Log.Put("xlsx の解析作業ができませんでした。")
            Log.Put("作業を強制終了します。")
            Exit Sub
        End Try


    End Sub
End Class
